const Node = require('./node')
const tokenTypes = require('./tokenTypes')

module.exports = class Parser {
  constructor(tokens) {
    this.tokens = tokens
  }

  parse() {
    let tree = []
    while (this.tokens.peek().type !== tokenTypes.EOF) {
      tree.push(this.expr())
    }
    return tree
  }

  expr() {
    this.tokens.eat(tokenTypes.OPAREN)

    let tokenTypesToEndTheLoop = [
      tokenTypes.OPAREN,
      tokenTypes.CPAREN,
      tokenTypes.EOF,
    ]

    let functionCallNode = new Node()
    functionCallNode.functionName = this.tokens.eat(tokenTypes.LITERAL).value
    functionCallNode.args = []
    functionCallNode.subtree = []

    while (!tokenTypesToEndTheLoop.includes(this.tokens.peek().type)) {
      if (this.tokens.peek().type === tokenTypes.ATTRIBUTE) {
        functionCallNode.args.push(this.attribute())
      }
    }

    this.tokens.eat(tokenTypes.CPAREN)

    return functionCallNode
  }

  attribute() {
    let attributeNode = new Node()
    attributeNode.attributeName = this.tokens.eat(tokenTypes.ATTRIBUTE).value
    attributeNode.attributeValue = this.string()
    return attributeNode
  }

  string() {
    this.tokens.eat(tokenTypes.QUOTE)
    let stringValue = this.tokens.eat(tokenTypes.LITERAL).value
    this.tokens.eat(tokenTypes.QUOTE)
    return stringValue
  }
}
